@@ -1,21 +1,35 @@
 import inspect
 from urllib.parse import urljoin
 
-from django.urls import URLResolver, URLPattern
-from rest_framework import viewsets, views
+from django.urls import URLPattern, URLResolver
+from rest_framework import views, viewsets
 from rest_framework.schemas.generators import BaseSchemaGenerator  # type: ignore
 from rest_framework.schemas.generators import EndpointEnumerator as BaseEndpointEnumerator
 
 from drf_spectacular.extensions import OpenApiViewExtension
 from drf_spectacular.plumbing import (
-    ComponentRegistry, error, alpha_operation_sorter, reset_generator_stats, build_root_object,
-    modify_for_versioning, operation_matches_version, warn, is_versioning_supported
+    ComponentRegistry, alpha_operation_sorter, build_root_object, error, is_versioning_supported,
+    modify_for_versioning, normalize_result_object, operation_matches_version,
+    reset_generator_stats, warn,
 )
 from drf_spectacular.settings import spectacular_settings
 
 
 class EndpointEnumerator(BaseEndpointEnumerator):
     def get_api_endpoints(self, patterns=None, prefix=''):
+        api_endpoints = self._get_api_endpoints(patterns, prefix)
+
+        for hook in spectacular_settings.PREPROCESSING_HOOKS:
+            api_endpoints = hook(endpoints=api_endpoints)
+
+        api_endpoints_deduplicated = {}
+        for path, path_regex, method, callback in api_endpoints:
+            if (path, method) not in api_endpoints_deduplicated:
+                api_endpoints_deduplicated[path, method] = (path, path_regex, method, callback)
+
+        return sorted(api_endpoints_deduplicated.values(), key=alpha_operation_sorter)
+
+    def _get_api_endpoints(self, patterns, prefix):
         """
         Return a list of all available API endpoints by inspecting the URL conf.
         Only modification the the DRF version is passing through the path_regex.
@@ -36,13 +50,13 @@ class EndpointEnumerator(BaseEndpointEnumerator):
                         api_endpoints.append(endpoint)
 
             elif isinstance(pattern, URLResolver):
-                nested_endpoints = self.get_api_endpoints(
+                nested_endpoints = self._get_api_endpoints(
                     patterns=pattern.url_patterns,
                     prefix=path_regex
                 )
                 api_endpoints.extend(nested_endpoints)
 
-        return sorted(api_endpoints, key=alpha_operation_sorter)
+        return api_endpoints
 
 
 class SchemaGenerator(BaseSchemaGenerator):
@@ -154,4 +168,4 @@ class SchemaGenerator(BaseSchemaGenerator):
         )
         for hook in spectacular_settings.POSTPROCESSING_HOOKS:
             result = hook(result=result, generator=self, request=request, public=public)
-        return result
+        return normalize_result_object(result)
