@@ -1,11 +1,12 @@
-import inspect
 import re
 import typing
 from operator import attrgetter
 
 import uritemplate
-from django.core import validators, exceptions as django_exceptions
+from django.core import exceptions as django_exceptions
+from django.core import validators
 from django.db import models
+from django.utils.translation import gettext_lazy as _
 from rest_framework import permissions, renderers, serializers
 from rest_framework.fields import _UnvalidatedField, empty
 from rest_framework.generics import GenericAPIView
@@ -16,21 +17,18 @@ from rest_framework.settings import api_settings
 from rest_framework.utils.model_meta import get_field_info
 from rest_framework.views import APIView
 
-from drf_spectacular.extensions import (
-    OpenApiSerializerFieldExtension, OpenApiSerializerExtension
+from drf_spectacular.authentication import OpenApiAuthenticationExtension
+from drf_spectacular.contrib import *  # noqa: F403, F401
+from drf_spectacular.extensions import OpenApiSerializerExtension, OpenApiSerializerFieldExtension
+from drf_spectacular.plumbing import (
+    ComponentRegistry, ResolvedComponent, anyisinstance, append_meta, build_array_type,
+    build_basic_type, build_choice_field, build_object_type, build_parameter_type, error,
+    follow_field_source, force_instance, get_doc, get_override, has_override, is_basic_type,
+    is_field, is_serializer, resolve_regex_path_parameter, safe_ref, warn,
 )
 from drf_spectacular.settings import spectacular_settings
-from drf_spectacular.contrib import *  # noqa: F403, F401
-from drf_spectacular.plumbing import (
-    build_basic_type, warn, anyisinstance, force_instance, is_serializer,
-    follow_field_source, is_field, is_basic_type, build_array_type,
-    ComponentRegistry, ResolvedComponent, build_parameter_type, error,
-    resolve_regex_path_parameter, safe_ref, has_override, get_override,
-    append_meta, build_object_type, build_choice_field,
-)
 from drf_spectacular.types import OpenApiTypes
 from drf_spectacular.utils import OpenApiParameter
-from drf_spectacular.authentication import OpenApiAuthenticationExtension
 
 
 class AutoSchema(ViewInspector):
@@ -53,6 +51,10 @@ class AutoSchema(ViewInspector):
         operation['operationId'] = self.get_operation_id()
         operation['description'] = self.get_description()
 
+        summary = self.get_summary()
+        if summary:
+            operation['summary'] = summary
+
         parameters = self._get_parameters()
         if parameters:
             operation['parameters'] = parameters
@@ -147,6 +149,18 @@ class AutoSchema(ViewInspector):
                 warn(f'could not resolve parameter annotation {parameter}. skipping.')
         return result
 
+    def _get_format_parameters(self):
+        parameters = []
+        formats = self.map_renderers('format')
+        if api_settings.URL_FORMAT_OVERRIDE and len(formats) > 1:
+            parameters.append(build_parameter_type(
+                name=api_settings.URL_FORMAT_OVERRIDE,
+                schema=build_basic_type(OpenApiTypes.STR),
+                location=OpenApiParameter.QUERY,
+                enum=formats
+            ))
+        return parameters
+
     def _get_parameters(self):
         def dict_helper(parameters):
             return {(p['name'], p['in']): p for p in parameters}
@@ -160,19 +174,25 @@ class AutoSchema(ViewInspector):
             **dict_helper(self._resolve_path_parameters(path_variables)),
             **dict_helper(self._get_filter_parameters()),
             **dict_helper(self._get_pagination_parameters()),
+            **dict_helper(self._get_format_parameters()),
         }
         # override/add @extend_schema parameters
         for key, parameter in override_parameters.items():
             parameters[key] = parameter
+
         return sorted(parameters.values(), key=lambda p: p['name'])
 
     def get_description(self):
         """ override this for custom behaviour """
         action_or_method = getattr(self.view, getattr(self.view, 'action', self.method.lower()), None)
-        view_doc = inspect.getdoc(self.view) or ''
-        action_doc = inspect.getdoc(action_or_method) or ''
+        view_doc = get_doc(self.view.__class__)
+        action_doc = get_doc(action_or_method)
         return action_doc or view_doc
 
+    def get_summary(self):
+        """ override this for custom behaviour """
+        return None
+
     def get_auth(self):
         """
         Obtains authentication classes and permissions from view. If authentication
@@ -260,15 +280,16 @@ class AutoSchema(ViewInspector):
         model = getattr(getattr(self.view, 'queryset', None), 'model', None)
         parameters = []
 
-        for idx, variable in enumerate(variables):
+        for variable in variables:
             schema = build_basic_type(OpenApiTypes.STR)
             description = ''
-            required = True
 
-            resolved_parameter = resolve_regex_path_parameter(self.path_regex, variable)
+            resolved_parameter = resolve_regex_path_parameter(
+                self.path_regex, variable, self.map_renderers('format'),
+            )
 
             if resolved_parameter:
-                schema, required = resolved_parameter['schema'], resolved_parameter['required']
+                schema = resolved_parameter['schema']
             elif not model:
                 warn(
                     f'could not derive type of path parameter "{variable}" because '
@@ -291,13 +312,12 @@ class AutoSchema(ViewInspector):
                         f'parameter with @extend_schema. defaulting to "string".'
                     )
 
-            parameters.append({
-                "name": variable,
-                "in": "path",
-                "required": required,
-                "description": description,
-                'schema': schema,
-            })
+            parameters.append(build_parameter_type(
+                name=variable,
+                location=OpenApiParameter.PATH,
+                description=description,
+                schema=schema
+            ))
 
         return parameters
 
@@ -357,7 +377,13 @@ class AutoSchema(ViewInspector):
         elif hasattr(models, model_field.get_internal_type()):
             # be graceful when the model field is not explicitly mapped to a serializer
             internal_type = getattr(models, model_field.get_internal_type())
-            field_cls = serializers.ModelSerializer.serializer_field_mapping[internal_type]
+            field_cls = serializers.ModelSerializer.serializer_field_mapping.get(internal_type)
+            if not field_cls:
+                warn(
+                    f'model field "{model_field.get_internal_type()}" has no mapping in '
+                    f'ModelSerializer. it may be a deprecated field. defaulting to "string"'
+                )
+                return build_basic_type(OpenApiTypes.STR)
             return self._map_serializer_field(field_cls(), direction)
         else:
             error(
@@ -384,13 +410,17 @@ class AutoSchema(ViewInspector):
 
         # nested serializer
         if isinstance(field, serializers.Serializer):
-            schema = self.resolve_serializer(field, direction).ref
-            return append_meta(schema, meta)
+            component = self.resolve_serializer(field, direction)
+            return append_meta(component.ref, meta) if component else None
 
         # nested serializer with many=True gets automatically replaced with ListSerializer
         if isinstance(field, serializers.ListSerializer):
-            schema = self.resolve_serializer(field.child, direction).ref
-            return append_meta(build_array_type(schema), meta)
+            if is_serializer(field.child):
+                component = self.resolve_serializer(field.child, direction)
+                return append_meta(build_array_type(component.ref), meta) if component else None
+            else:
+                schema = self._map_serializer_field(field.child, direction)
+                return append_meta(build_array_type(schema), meta)
 
         # Related fields.
         if isinstance(field, serializers.ManyRelatedField):
@@ -533,7 +563,7 @@ class AutoSchema(ViewInspector):
 
         if isinstance(field, serializers.ReadOnlyField):
             # direct source from the serializer
-            assert field.source_attrs, 'ReadOnlyField needs a proper source'
+            assert field.source_attrs, f'ReadOnlyField "{field}" needs a proper source'
             target = follow_field_source(field.parent.Meta.model, field.source_attrs)
 
             if callable(target):
@@ -541,7 +571,8 @@ class AutoSchema(ViewInspector):
             elif isinstance(target, models.Field):
                 schema = self._map_model_field(target, direction)
             else:
-                assert False, 'ReadOnlyField target must be property or model field'
+
+                assert False, f'ReadOnlyField target "{field}" must be property or model field'
             return append_meta(schema, meta)
 
         # DRF was not able to match the model field to an explicit SerializerField and therefore
@@ -550,7 +581,7 @@ class AutoSchema(ViewInspector):
             schema = self._map_model_field(field.model_field, direction)
             return append_meta(schema, meta)
 
-        warn(f'could not resolve serializer field {field}. defaulting to "string"')
+        warn(f'could not resolve serializer field "{field}". defaulting to "string"')
         return append_meta(build_basic_type(OpenApiTypes.STR), meta)
 
     def _map_min_max(self, field, content):
@@ -592,9 +623,20 @@ class AutoSchema(ViewInspector):
                 if prop_name in required:
                     required.remove(prop_name)
 
+        # remove empty listing as it violates schema specification
+        if 'required' in schema and not required:
+            del schema['required']
         return schema
 
     def _get_serializer_field_meta(self, field):
+        if not isinstance(field, serializers.Field):
+            warn(
+                f'unable to extract field metadata from field "{field}" because it appears '
+                f'to be neither a Field nor a Serializer. Proper handling may require an '
+                f'Extension or @extend_serializer_field. Skipping metadata extraction. '
+            )
+            return {}
+
         meta = {}
         if field.read_only:
             meta['readOnly'] = True
@@ -619,10 +661,19 @@ class AutoSchema(ViewInspector):
         for field in serializer.fields.values():
             if isinstance(field, serializers.HiddenField):
                 continue
+            if field.field_name in get_override(serializer, 'exclude_fields', []):
+                continue
 
             schema = self._map_serializer_field(field, direction)
+            # skip field if there is no schema for the direction
+            if not schema:
+                continue
 
-            if field.required or schema.get('readOnly'):
+            add_to_required = (
+                field.required
+                or (schema.get('readOnly') and not spectacular_settings.COMPONENT_NO_READ_ONLY_REQUIRED)
+            )
+            if add_to_required:
                 required.add(field.field_name)
 
             self._map_field_validators(field, schema)
@@ -636,7 +687,7 @@ class AutoSchema(ViewInspector):
         return build_object_type(
             properties=properties,
             required=required,
-            description=inspect.getdoc(serializer),
+            description=get_doc(serializer.__class__),
         )
 
     def _map_field_validators(self, field, schema):
@@ -700,14 +751,12 @@ class AutoSchema(ViewInspector):
     def map_parsers(self):
         return list(map(attrgetter('media_type'), self.view.parser_classes))
 
-    def map_renderers(self):
-        media_types = []
-        for renderer in self.view.renderer_classes:
-            # BrowsableAPIRenderer not relevant to OpenAPI spec
-            if renderer == renderers.BrowsableAPIRenderer:
-                continue
-            media_types.append(renderer.media_type)
-        return media_types
+    def map_renderers(self, attribute):
+        assert attribute in ['media_type', 'format']
+        return list(dict.fromkeys([
+            getattr(r, attribute) for r in self.view.renderer_classes
+            if r != renderers.BrowsableAPIRenderer
+        ]))
 
     def _get_serializer(self):
         view = self.view
@@ -751,12 +800,15 @@ class AutoSchema(ViewInspector):
 
         request_body_required = False
         if isinstance(serializer, serializers.ListSerializer):
-            component = self.resolve_serializer(serializer.child, 'request')
-            schema = build_array_type(component.ref)
+            if is_serializer(serializer.child):
+                component = self.resolve_serializer(serializer.child, 'request')
+                schema = build_array_type(component.ref)
+            else:
+                schema = build_array_type(self._map_serializer_field(serializer.child, 'request'))
             request_body_required = True
         elif is_serializer(serializer):
             component = self.resolve_serializer(serializer, 'request')
-            if not component:
+            if not component.schema:
                 # serializer is empty so skip content enumeration
                 return None
             schema = component.ref
@@ -795,7 +847,7 @@ class AutoSchema(ViewInspector):
 
         if is_serializer(response_serializers) or is_basic_type(response_serializers):
             if self.method == 'DELETE':
-                return {'204': {'description': 'No response body'}}
+                return {'204': {'description': _('No response body')}}
             return {'200': self._get_response_for_code(response_serializers)}
         elif isinstance(response_serializers, dict):
             # custom handling for overriding default return codes with @extend_schema
@@ -810,20 +862,23 @@ class AutoSchema(ViewInspector):
                 f'defaulting to generic free-form object.'
             )
             schema = build_basic_type(OpenApiTypes.OBJECT)
-            schema['description'] = 'Unspecified response body'
+            schema['description'] = _('Unspecified response body')
             return {'200': self._get_response_for_code(schema)}
 
     def _get_response_for_code(self, serializer):
         serializer = force_instance(serializer)
 
         if not serializer:
-            return {'description': 'No response body'}
+            return {'description': _('No response body')}
         elif isinstance(serializer, serializers.ListSerializer):
-            schema = self.resolve_serializer(serializer.child, 'response').ref
+            if is_serializer(serializer.child):
+                schema = self.resolve_serializer(serializer.child, 'response').ref
+            else:
+                schema = self._map_serializer_field(serializer.child, 'response')
         elif is_serializer(serializer):
             component = self.resolve_serializer(serializer, 'response')
-            if not component:
-                return {'description': 'No response body'}
+            if not component.schema:
+                return {'description': _('No response body')}
             schema = component.ref
         elif is_basic_type(serializer):
             schema = build_basic_type(serializer)
@@ -837,7 +892,7 @@ class AutoSchema(ViewInspector):
                 f'generic free-form object.'
             )
             schema = build_basic_type(OpenApiTypes.OBJECT)
-            schema['description'] = 'Unspecified response body'
+            schema['description'] = _('Unspecified response body')
 
         if self._is_list_view(serializer) and not get_override(serializer, 'many') is False:
             schema = build_array_type(schema)
@@ -847,7 +902,7 @@ class AutoSchema(ViewInspector):
 
         return {
             'content': {
-                mt: {'schema': schema} for mt in self.map_renderers()
+                mt: {'schema': schema} for mt in self.map_renderers('media_type')
             },
             # Description is required by spec, but descriptions for each response code don't really
             # fit into our model. Description is therefore put into the higher level slots.
@@ -881,7 +936,12 @@ class AutoSchema(ViewInspector):
         return name
 
     def resolve_serializer(self, serializer, direction) -> ResolvedComponent:
-        assert is_serializer(serializer)
+        assert is_serializer(serializer), (
+            f'internal assumption violated because we expected a serializer here and instead '
+            f'got a "{serializer}". This may be the result of another app doing some unexpected '
+            f'magic or an invalid internal call. Feel free to report this as a bug at '
+            f'https://github.com/tfranzel/drf-spectacular/issues '
+        )
         serializer = force_instance(serializer)
 
         component = ResolvedComponent(
