@@ -3,23 +3,24 @@ import inspect
 import json
 import sys
 from abc import ABCMeta
-from collections import defaultdict, OrderedDict
+from collections import OrderedDict, defaultdict
 from collections.abc import Hashable, Iterable
 from decimal import Decimal
 from enum import Enum
-from typing import List, Type, Optional, TypeVar, Union, Generic, DefaultDict
+from typing import DefaultDict, Generic, List, Optional, Type, TypeVar, Union
 
-import inflection
 import uritemplate
 from django import __version__ as DJANGO_VERSION
+from django.apps import apps
 from django.urls.resolvers import _PATH_PARAMETER_COMPONENT_RE, get_resolver  # type: ignore
+from django.utils.functional import Promise
 from django.utils.module_loading import import_string
-from rest_framework import fields, serializers, versioning, exceptions
+from rest_framework import exceptions, fields, mixins, serializers, versioning
 from uritemplate import URITemplate
 
 from drf_spectacular.settings import spectacular_settings
 from drf_spectacular.types import (
-    OPENAPI_TYPE_MAPPING, PYTHON_TYPE_MAPPING, DJANGO_PATH_CONVERTER_MAPPING, OpenApiTypes
+    DJANGO_PATH_CONVERTER_MAPPING, OPENAPI_TYPE_MAPPING, PYTHON_TYPE_MAPPING, OpenApiTypes,
 )
 from drf_spectacular.utils import OpenApiParameter
 
@@ -125,12 +126,46 @@ def has_override(obj, prop):
     return True
 
 
-def get_override(obj, prop):
+def get_override(obj, prop, default=None):
     if not has_override(obj, prop):
-        return None
+        return default
     return obj._spectacular_annotation[prop]
 
 
+def get_lib_doc_excludes():
+    # do not import on package level due to potential import recursion when loading
+    # extensions as recommended:  USER's settings.py -> USER EXTENSIONS -> extensions.py
+    # -> plumbing.py -> DRF views -> DRF DefaultSchema -> openapi.py - plumbing.py -> Loop
+    from rest_framework import generics, views, viewsets
+    return [
+        views.APIView,
+        *[getattr(serializers, c) for c in dir(serializers) if c.endswith('Serializer')],
+        *[getattr(viewsets, c) for c in dir(viewsets) if c.endswith('ViewSet')],
+        *[getattr(generics, c) for c in dir(generics) if c.endswith('APIView')],
+        *[getattr(mixins, c) for c in dir(mixins) if c.endswith('Mixin')],
+    ]
+
+
+def get_doc(obj):
+    """ get doc string with fallback on obj's base classes (ignoring DRF documentation). """
+    if not inspect.isclass(obj):
+        return inspect.getdoc(obj) or ''
+
+    def safe_index(lst, item):
+        try:
+            return lst.index(item)
+        except ValueError:
+            return float("inf")
+
+    lib_barrier = min(
+        safe_index(obj.__mro__, c) for c in spectacular_settings.GET_LIB_DOC_EXCLUDES()
+    )
+    for cls in obj.__mro__[:lib_barrier]:
+        if cls.__doc__:
+            return inspect.cleandoc(cls.__doc__)
+    return ''
+
+
 def build_basic_type(obj):
     """
     resolve either enum or actual type and yield schema template for modification
@@ -198,8 +233,8 @@ def build_parameter_type(
         schema['explode'] = explode
     if style is not None:
         schema['style'] = style
-    if enum is not None:
-        schema['schema']['enum'] = enum
+    if enum:
+        schema['schema']['enum'] = sorted(enum)
     return schema
 
 
@@ -349,6 +384,12 @@ def alpha_operation_sorter(endpoint):
         'PATCH': 3,
         'DELETE': 4
     }.get(method, 5)
+
+    # Sort foo{arg} after foo/, but before foo/bar
+    if path.endswith('/'):
+        path = path[:-1] + ' '
+    path = path.replace('{', '!')
+
     return path, method_priority
 
 
@@ -363,7 +404,7 @@ class ResolvedComponent:
         self.object = object
 
     def __bool__(self):
-        return bool(self.schema)
+        return bool(self.name and self.type and self.object)
 
     @property
     def key(self):
@@ -449,6 +490,12 @@ class OpenApiGeneratorExtension(Generic[T], metaclass=ABCMeta):
         try:
             cls.target_class = import_string(cls.target_class)
         except ImportError:
+            installed_apps = apps.app_configs.keys()
+            if any(cls.target_class.startswith(app + '.') for app in installed_apps):
+                warn(
+                    f'registered extensions {cls.__name__} for "{cls.target_class}" '
+                    f'has an installed app but target class was not found.'
+                )
             cls.target_class = None
 
     @classmethod
@@ -516,111 +563,31 @@ def list_hash(lst):
     return hashlib.blake2b(json.dumps(list(lst), sort_keys=True).encode()).hexdigest()
 
 
-def postprocess_schema_enums(result, generator, **kwargs):
-    """
-    simple replacement of Enum/Choices that globally share the same name and have
-    the same choices. Aids client generation to not generate a separate enum for
-    every occurrence. only takes effect when replacement is guaranteed to be correct.
-    """
-    def iter_prop_containers(schema):
-        if isinstance(schema, list):
-            for item in schema:
-                yield from iter_prop_containers(item)
-        elif isinstance(schema, dict):
-            if schema.get('properties'):
-                yield schema['properties']
-            yield from iter_prop_containers(schema.get('oneOf', []))
-            yield from iter_prop_containers(schema.get('allOf', []))
-
-    schemas = result.get('components', {}).get('schemas', {})
-
-    overrides = load_enum_name_overrides()
-
-    hash_mapping = defaultdict(set)
-    # collect all enums, their names and choice sets
-    for props in iter_prop_containers(list(schemas.values())):
-        for prop_name, prop_schema in props.items():
-            if 'enum' not in prop_schema:
-                continue
-            hash_mapping[prop_name].add(list_hash(prop_schema['enum']))
-
-    # traverse all enum properties and generate a name for the choice set. naming collisions
-    # are resolved and a warning is emitted. giving a choice set multiple names is technically
-    # correct but potentially unwanted. also emit a warning there to make the user aware.
-    enum_name_mapping = {}
-    for prop_name, prop_hash_set in hash_mapping.items():
-        for prop_hash in prop_hash_set:
-            if prop_hash in overrides:
-                enum_name = overrides[prop_hash]
-            elif len(prop_hash_set) == 1:
-                enum_name = f'{inflection.camelize(prop_name)}Enum'
-            else:
-                enum_name = f'{inflection.camelize(prop_name)}{prop_hash[:3].capitalize()}Enum'
-                warn(
-                    f'automatic enum naming encountered a collision for field "{prop_name}". the '
-                    f'same name has been used for multiple choice sets. the collision was resolved '
-                    f'with {enum_name}. add an entry to ENUM_NAME_OVERRIDES to fix the naming.'
-                )
-            if enum_name_mapping.get(prop_hash, enum_name) != enum_name:
-                warn(
-                    f'encountered multiple names for the same choice set ({enum_name}). this '
-                    f'may be unwanted even though the generated schema is technically correct. '
-                    f'add an entry to ENUM_NAME_OVERRIDES to fix the naming.'
-                )
-                del enum_name_mapping[prop_hash]
-            else:
-                enum_name_mapping[prop_hash] = enum_name
-            enum_name_mapping[(prop_hash, prop_name)] = enum_name
-
-    # replace all enum occurrences with a enum schema component. cut out the
-    # enum, replace it with a reference and add a corresponding component.
-    for props in iter_prop_containers(list(schemas.values())):
-        for prop_name, prop_schema in props.items():
-            if 'enum' not in prop_schema:
-                continue
-
-            prop_hash = list_hash(prop_schema['enum'])
-            # when choice sets are reused under multiple names, the generated name cannot be
-            # resolved from the hash alone. fall back to prop_name and hash for resolution.
-            enum_name = enum_name_mapping.get(prop_hash) or enum_name_mapping[prop_hash, prop_name]
-
-            enum_schema = {k: v for k, v in prop_schema.items() if k in ['type', 'enum']}
-            prop_schema = {k: v for k, v in prop_schema.items() if k not in ['type', 'enum']}
-
-            component = ResolvedComponent(
-                name=enum_name,
-                type=ResolvedComponent.SCHEMA,
-                schema=enum_schema,
-                object=enum_name,
-            )
-            if component not in generator.registry:
-                generator.registry.register(component)
-            prop_schema.update(component.ref)
-            props[prop_name] = safe_ref(prop_schema)
-
-    # sort again with additional components
-    result['components'] = generator.registry.build(spectacular_settings.APPEND_COMPONENTS)
-    return result
-
-
-def resolve_regex_path_parameter(path_regex, variable):
+def resolve_regex_path_parameter(path_regex, variable, available_formats):
     """
     convert django style path parameters to OpenAPI parameters.
     TODO also try to handle regular grouped regex parameters
     """
     for match in _PATH_PARAMETER_COMPONENT_RE.finditer(path_regex):
         converter, parameter = match.group('converter'), match.group('parameter')
+        enum_values = None
 
         if converter and converter.startswith('drf_format_suffix_'):
-            converter = 'drf_format_suffix'  # remove appended options
+            explicit_formats = converter[len('drf_format_suffix_'):].split('_')
+            enum_values = [
+                f'.{suffix}' for suffix in explicit_formats if suffix in available_formats
+            ]
+            converter = 'drf_format_suffix'
+        elif converter == 'drf_format_suffix':
+            enum_values = [f'.{suffix}' for suffix in available_formats]
 
         if parameter == variable and converter in DJANGO_PATH_CONVERTER_MAPPING:
-            return {
-                'name': parameter,
-                'schema': build_basic_type(DJANGO_PATH_CONVERTER_MAPPING[converter]),
-                'in': OpenApiParameter.PATH,
-                'required': False if converter == 'drf_format_suffix' else True,
-            }
+            return build_parameter_type(
+                name=parameter,
+                schema=build_basic_type(DJANGO_PATH_CONVERTER_MAPPING[converter]),
+                location=OpenApiParameter.PATH,
+                enum=enum_values,
+            )
 
     return None
 
@@ -662,3 +629,14 @@ def modify_for_versioning(patterns, method, path, view, requested_version):
         mocked_request.resolver_match = get_resolver(tuple(patterns)).resolve(path)
 
     return path
+
+
+def normalize_result_object(result):
+    """ resolve non-serializable objects like lazy translation strings and OrderedDict """
+    if isinstance(result, dict) or isinstance(result, OrderedDict):
+        return {k: normalize_result_object(v) for k, v in result.items()}
+    if isinstance(result, list) or isinstance(result, tuple):
+        return [normalize_result_object(v) for v in result]
+    if isinstance(result, Promise):
+        return str(result)
+    return result
