@@ -11,9 +11,14 @@ SPECTACULAR_DEFAULTS: Dict[str, Any] = {
 
     # Schema generation parameters to influence how components are constructed.
     # Some schema features might not translate well to your target.
-    # Demultiplexing components might help alleviate those issues.
+    # Demultiplexing/modifying components might help alleviate those issues.
+    #
+    # Create separate components for PATCH endpoints (without required list)
     'COMPONENT_SPLIT_PATCH': True,
+    # Split components into request and response parts where appropriate
     'COMPONENT_SPLIT_REQUEST': False,
+    # Aid client generator targets that have trouble with read-only properties.
+    "COMPONENT_NO_READ_ONLY_REQUIRED": False,
 
     # Configuration for serving the schema with SpectacularAPIView
     'SERVE_URLCONF': None,
@@ -30,12 +35,21 @@ SPECTACULAR_DEFAULTS: Dict[str, Any] = {
     # Postprocessing functions that run at the end of schema generation.
     # must satisfy interface result = hook(generator, request, public, result)
     'POSTPROCESSING_HOOKS': [
-        'drf_spectacular.plumbing.postprocess_schema_enums'
+        'drf_spectacular.hooks.postprocess_schema_enums'
     ],
 
+    # Preprocessing functions that run before schema generation.
+    # must satisfy interface result = hook(endpoints=result) where result
+    # is a list of Tuples (path, path_regex, method, callback).
+    # Example: 'drf_spectacular.hooks.preprocess_exclude_path_format'
+    'PREPROCESSING_HOOKS': [],
+
     # enum name overrides. dict with keys "YourEnum" and their choice values "field.choices"
     'ENUM_NAME_OVERRIDES': {},
 
+    # function that returns a list of all classes that should be excluded from doc string extraction
+    'GET_LIB_DOC_EXCLUDES': 'drf_spectacular.plumbing.get_lib_doc_excludes',
+
     # General schema metadata. Refer to spec for valid inputs
     # https://github.com/OAI/OpenAPI-Specification/blob/master/versions/3.0.3.md#openapi-object
     'TITLE': '',
@@ -69,6 +83,8 @@ IMPORT_STRINGS = [
     'DEFAULT_GENERATOR_CLASS',
     'SERVE_PERMISSIONS',
     'POSTPROCESSING_HOOKS',
+    'PREPROCESSING_HOOKS',
+    'GET_LIB_DOC_EXCLUDES',
 ]
 
 spectacular_settings = APISettings(
